

class WordRawData():
    """
    Represents the raw-data generated by (py)tesseract
    """
    def __init__(self, p_data):
        self.level = int(p_data[0])
        self.page = int(p_data[1])
        self.block = int(p_data[2])
        self.paragraph = int(p_data[3])
        self.line = int(p_data[4])
        self.word = int(p_data[5])
        self.left = int(p_data[6])
        self.top = int(p_data[7])
        self.width = int(p_data[8])
        self.height = int(p_data[9])
        self.confidence = int(p_data[10])
        if len(p_data) >= 12:
            self.text = p_data[11].replace("—", "-").replace(" ", "")
        else:
            self.text = ""

    def to_json(self):
        return [
            self.level,
            self.page,
            self.block,
            self.paragraph,
            self.line,
            self.word,
            self.left,
            self.top,
            self.width,
            self.height,
            self.confidence,
            self.text
        ]

    @staticmethod
    def from_json(p_json):
        return WordRawData(p_json)


class Word():
    """
    Represents a word.
    """
    def __init__(self, p_data):
        self.raw_data = WordRawData(p_data)
        self.literal = self.raw_data.text
        self.index = self.raw_data.word
        self.block = None
        self.line = None

    def to_json(self):
        return self.raw_data.to_json()

    @staticmethod
    def from_json(p_json):
        return Word(p_json)

    def __str__(self):
        return self.literal


class Line():
    def __init__(self, p_index):
        self.block = None
        self.index = p_index
        self.word_list = []

    def insert_word(self, p_word):
        self.word_list.append(p_word)
        p_word.line = self
        p_word.block = self.block

    def to_json(self):
        words_as_json = []
        for word in self.word_list:
            words_as_json.append(word.to_json())
        return [self.index, words_as_json]

    @staticmethod
    def from_json(p_json):
        index = p_json[0]
        line = Line(index)
        for word_as_json in p_json[1]:
            word = Word.from_json(word_as_json)
            word.line = line
            word.block = line.block
            line.word_list.append(word)
        return line

    def __str__(self):
        string = ""
        for word in self.word_list:
            string += str(word) + " "
        return string

    def get_word_list(self):
        return self.word_list

    def get_word_at(self, p_index):
        return self.word_list[p_index]

    def __repr__(self):
        return str(self)


class Block():
    def __init__(self, p_index):
        self.page = None
        self.index = p_index
        self.line_list = {}

    def _add_line(self, p_index):
        line = Line(p_index)
        line.block = self
        self.line_list[p_index] = line
        return line

    def insert_word(self, p_word):
        index = p_word.raw_data.line
        line = None
        if index in self.line_list:
            line = self.line_list[index]
        else:
            line = self._add_line(index)
        line.insert_word(p_word)
        self.line_list[index] = line

    def get_line_list(self):
        return self.line_list.values()

    def get_line_at(self, p_index):
        return self.line_list[p_index]

    def to_json(self):
        lines_as_json = []
        for line in self.line_list.values():
            lines_as_json.append(line.to_json())
        return [self.index, lines_as_json]

    @staticmethod
    def from_json(p_json):
        index = p_json[0]
        block = Block(index)
        for line_as_json in p_json[1]:
            line = Line.from_json(line_as_json)
            line.block = block
            block.line_list[line.index] = line
        return block

    def __str__(self):
        string = ""
        for line in self.line_list.values():
            string += str(line) + "\n"
        return string
    

class Page():
    def __init__(self, p_index=0):
        self.index = p_index
        self.block_list = {}

    def _add_block(self, p_index):
        block = Block(p_index)
        block.page = self
        self.block_list[p_index] = block
        return block

    def insert_word(self, p_word):
        index = p_word.raw_data.block
        block = None
        if index in self.block_list:
            block = self.block_list[index]
        else:
            block = self._add_block(index)
        block.insert_word(p_word)
        self.block_list[index] = block

    def get_block_list(self):
        return self.block_list.values()
    
    def get_block_at(self, p_index):
        return self.block_list[p_index]

    def to_json(self):
        blocks_as_json = []
        for block in self.block_list.values():
            blocks_as_json.append(block.to_json())
        return [self.index, blocks_as_json]

    @staticmethod
    def from_json(p_json):
        index = p_json[0]
        page = Page(index)
        for block_as_json in p_json[1]:
            block = Block.from_json(block_as_json)
            block.page = page
            page.block_list[block.index] = block
        return page

    def __str__(self):
        string = ""
        for block in self.block_list.values():
            string += str(block) + "\n\n\n"
        return string

    